<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 09/28/2003 *
 * ===================                          Last change: 08/18/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-list_user.php                               *
 * -------------------------------------------------------------------- *
 * Short description : List all or partial members                      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Alle oder nur bestimmte Mitglieder auflisten     *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if ((!defined('__SECURITY')) || (!isAdmin())) {
	die();
} // END - if

// Add description as navigation point
addYouAreHereLink('admin', __FILE__);

// Init variables
$moreColumns = '';

// Add lock reason?
if (isExtensionInstalledAndNewer('user', '0.3.5')) {
	// Add them...
	$moreColumns .= ', UNIX_TIMESTAMP(`lock_timestamp`) AS `lock_timestamp`';
} // END - if

// Is the extension 'country' installed?
if (isExtensionActive('country')) {
	// Add country code
	$moreColumns .= ',`country_code`';
} // END - if

// Init unset data
if (!isGetRequestElementSet('letter')) { setGetRequestElement('letter', '');       }
if (!isGetRequestElementSet('sortby')) { setGetRequestElement('sortby', 'userid'); }
if (!isGetRequestElementSet('page'))   { setGetRequestElement('page'  , 1);        }

// Set base HTML
$base = '[<a href="{%url=modules.php?module=admin';

if (isGetRequestElementSet('userid')) {
	// Secure the user id
	$userid = bigintval(getRequestElement('userid'));

	// Does the account exists?
	$result_user = sqlQueryEscaped('SELECT
	`userid`,
	`country`,
	`email`,
	`birth_day`,
	`birth_month`,
	`birth_year`,
	`last_online`,
	`used_points`,
	`emails_sent`,
	`joined`,
	`last_update`,
	`last_profile_sent`,
	`notified`,
	`emails_received`,
	`mails_confirmed`
	' . $moreColumns . '
FROM
	`{?_MYSQL_PREFIX?}_user_data`
WHERE
	`userid`=%s
LIMIT 1',
		array($userid), __FILE__, __LINE__);

	// User found?
	if (sqlNumRows($result_user) == 1) {
		// Account found
		$content  = sqlFetchArray($result_user);

		// Get count/sum of refs, selected categories, unconfirmed mails
		// @TODO Refacture these old link generation to functions
		$content['refs']  = countSumTotalData($userid, 'refsystem', 'counter');
		$content['cats']  = countSumTotalData($userid, 'user_cats', 'id', 'userid', TRUE);
		$content['links'] = getTotalUnconfirmedMails($userid);

		// Add links to the numbers
		if ($content['links'] > 0) {
			$content['links'] = $base . '&amp;what=list_links&amp;userid=' . $userid . '%}">' . $content['links'] . '</a>]';
		} // END - if
		if ($content['refs']  > 0) {
			$content['refs']  = $base . '&amp;what=list_refs&amp;userid=' . $userid . '%}">{%pipe,translateComma=' . $content['refs'] . '%}</a>]';
		} // END - if
		if ($content['cats']  > 0) {
			$content['cats']  = $base . '&amp;what=list_user_cats&amp;userid=' . $userid . '%}">{%pipe,translateComma=' . $content['cats'] . '%}</a>]';
		} // END - if

		// Calculate timestamp for birthday
		// @TODO Refacture birth_foo to avoid uni* timetstamps as older members cannot be greeded correctly
		$stamp = mktime(0, 0, 0, $content['birth_month'], $content['birth_day'], $content['birth_year']);

		// Is this above zero?
		if ($stamp > 0) {
			// Then use it
			$content['birthday'] = generateDateTime($stamp, '2');
		} else {
			// Zero or below so set zero!
			$content['birthday'] = generateDateTime(0, '2');
		}

		// Prepare data for template
		$content['email_link']        = generateEmailLink($content['email'], 'user_data');
		$content['last_online']       = generateDateTime($content['last_online'], '2');
		$content['joined']            = generateDateTime($content['joined'], '2');
		$content['last_update']       = generateDateTime($content['last_update'], '2');
		$content['last_profile_sent'] = generateDateTime($content['last_profile_sent'], '2');
		$content['locked_points']     = countSumTotalData($content['userid'], 'user_points', 'locked_points') + countSumTotalData($content['userid'], 'user_points', 'locked_order_points');

		// Prepare array for filter
		$data = array(
			'userid' => $content['userid'],
			'points' => 0
		);

		// Run the filter chain
		$data = runFilterChain('get_own_points', $data);

		// Add more data
		$content['own_points']     = $data['points'];
		$content['lock_timestamp'] = generateDateTime($content['lock_timestamp'], '2');

		// Link to email_details if some are sent
		if ($content['emails_sent'] > 0) {
			$content['emails_sent'] = $base . '&amp;what=email_details&amp;userid=' . $content['userid'] . '%}">{%pipe,translateComma=' . $content['emails_sent'] . '%}</a>]';
		} // END - if

		// Clickrate
		$content['click_rate'] = calculatePercentageRate($content['mails_confirmed'], $content['emails_received']);

		// Is the extension 'country' installed?
		if (isExtensionActive('country')) {
			// Then overwrite country information
			$content['country'] = generateCountryInfo($content['country_code']);
		} elseif ($content['country'] == '0') {
			// Zero id???
			$content['country'] = '???';
		}

		// Add userid
		$content['userid'] = $userid;

		// Load user-details template
		loadTemplate('admin_user_details', FALSE, $content);
	} else {
		// Account does not exist!
		displayMessage('{%message,ADMIN_MEMBER_404=' . $userid . '%}');
	}

	// Free the result
	sqlFreeResult($result_user);
} else {
	$whereStatement = '';
	if (isGetRequestElementSet('letter')) {
		// List only persons w
		$whereStatement = " WHERE `family` LIKE '".getRequestElement('letter') . "%%'";
	} // END - if

	// Parse the status or mode parameter
	if (isGetRequestElementSet('status')) {
		// Is a WHERE statement already there?
		if (!empty($whereStatement)) {
			// Then append the status column
			$whereStatement .= sprintf(" AND `status`='%s'", sqlEscapeString(strtoupper(getRequestElement('status'))));
		} else {
			// Start a new one
			$whereStatement = sprintf(" WHERE `status`='%s'", sqlEscapeString(strtoupper(getRequestElement('status'))));
		}

		// Exclude other users
		$whereStatement = runFilterChain('user_exclusion_sql', ' ' . $whereStatement);
	} elseif (isGetRequestElementSet('do')) {
		// Choose what we need to list
		switch (getRequestElement('do')) {
			case 'norefs': // Users w/o refs
				if (!empty($whereStatement)) {
					// Add AND statement
					$whereStatement .= ' AND (`refid`=0 OR `refid` IS NULL)';
				} else {
					// Add WHERE statement
					$whereStatement = ' WHERE (`refid`=0 OR `refid` IS NULL)';
				}
				break;

			case 'random_refid': // Users available for random referral id
				if (!empty($whereStatement)) {
					// Add AND statement
					$whereStatement .= ' AND `rand_confirmed` >= {?user_min_confirmed?}';
				} else {
					// Add WHERE statement
					$whereStatement = ' WHERE `rand_confirmed` >= {?user_min_confirmed?}';
				}
				break;

			case 'testers': // Test accounts
				// Add statement through filter
				$whereStatement = runFilterChain('user_inclusion_sql', $whereStatement);
				break;

			default: // Invalid list mode
				reportBug(__FILE__, __LINE__, sprintf('Invalid do %s detected.', sqlEscapeString(getRequestElement('do'))));
				break;
		} // END - switch

		// If it is not 'testers', add exclusion SQL
		if (getRequestElement('do') != 'testers') {
			// Exclude tester users
			$whereStatement = runFilterChain('user_exclusion_sql', ' ' . $whereStatement);
		} // END - if
	} // END - if

	// Generate master query string
	$sql = sprintf('SELECT
	`userid`,
	`email`,
	`emails_sent`,
	`mails_confirmed`,
	`emails_received`
	' . $moreColumns . '
FROM
	`{?_MYSQL_PREFIX?}_user_data`
' . $whereStatement . '
ORDER BY
	`%s` ASC',
		getRequestElement('sortby')
	);

	// Prepare SQL and run it
	$result_master = sqlQuery($sql, __FILE__, __LINE__);

	// Is the user limit zero?
	if (getUserLimit() == '0') {
		// Then set it to default ...
		setConfigEntry('user_limit', 100);

		// ... but output warning
		displayMessage('{--EXTENSION_WARNING_USER_LIMIT--}');
	} // END - if

	// Calculate page count (0.5 fixes a bug with page count)
	$numPages = round(sqlNumRows($result_master) / getUserLimit() + 0.5);

	if (!isGetRequestElementSet('page'))   setGetRequestElement('page'  , 1);
	if (!isGetRequestElementSet('offset')) setGetRequestElement('offset', getUserLimit());

	// Add limitation to SQL string and run him again
	$sql .= ' LIMIT ' . (getRequestElement('offset') * getRequestElement('page') - getRequestElement('offset')) . ', ' . getRequestElement('offset');
	$result = sqlQuery($sql, __FILE__, __LINE__);

	// Are there some entries?
	if (!ifSqlHasZeroNumRows($result_master)) {
		// Sorting links and number of pages
		$templateContent['sort_links'] = addSortLinks(getRequestElement('letter'), getRequestElement('sortby'));
		$templateContent['num_pages'] = $numPages;

		// Column with nickname when nickname extension is present
		// @TODO Rewrite this into a filter
		$templateContent['nickname_th'] = '';
		if (isExtensionActive('nickname')) {
			// Nickname extension found
			$templateContent['nickname_th'] = '<td class="header_column bottom" align="center">{--NICKNAME--}</td>';
		} // END - if

		// Load all users
		$OUT = '';
		while ($content = sqlFetchArray($result)) {
			// Merge more data in
			$content = merge_array($content, $templateContent);

			// Get number of unconfirmed mails
			$content['links'] = getTotalUnconfirmedMails($content['userid']);
			if ($content['links'] > 0) {
				$content['links'] = $base . '&amp;what=list_links&amp;userid=' . $content['userid'] . '%}">{%pipe,translateComma=' . $content['links'] . '%}</a>]';
			} // END - if

			// Set link to sent mails if present
			if ($content['emails_sent'] > 0) {
				$content['emails_sent'] = $base . '&amp;what=email_details&amp;userid=' . $content['userid'] . '%}">{%pipe,translateComma=' . $content['emails_sent'] . '%}</a>]';
			} // END - if

			// Click rate
			$content['click_rate'] = calculatePercentageRate($content['mails_confirmed'], $content['emails_received']);

			// Transfer data to array
			$content['email']          = '[<a href="' . generateEmailLink($content['email'], 'user_data') . '">' . $content['email'] . '</a>]';
			$content['lock_timestamp'] = generateDateTime($content['lock_timestamp'], '2');
			$content['refs']           = countSumTotalData($content['userid'], 'user_refs', 'id', 'userid', TRUE);

			// Prepare array for filter
			$data = array(
				'userid' => $content['userid'],
				'points' => 0
			);

			// Run the filter chain
			$data = runFilterChain('get_own_points', $data);

			// Copy points
			$content['own_points'] = $data['points'];

			// Prepare array for filter again
			$data = array(
				'userid' => $content['userid'],
				'points' => 0
			);

			// Run the filter chain
			$data = runFilterChain('get_locked_points', $data);

			// Copy points
			$content['locked_points']  = $data['points'];

			// If we have at least one referral, make it clickable to referral list
			if ($content['refs'] > 0) {
				$content['refs'] = $base . '&amp;what=list_refs&amp;userid=' . $content['userid'] . '%}">{%pipe,translateComma=' . $content['refs'] . '%}</a>]';
			} // END - if

			// Is the extension 'country' installed?
			// @TODO Rewrite this into a filter
			if (isExtensionActive('country')) {
				// Then overwrite country information
				$content['country'] = generateCountryInfo($content['country_code']);
			} elseif ($content['country'] == '') {
				// Zero id???
				$content['country'] = '???';
			}

			// Load row template and switch colors
			$OUT .= loadTemplate('admin_list_user_row', TRUE, $content);
		} // END - while

		// Free memory
		sqlFreeResult($result);

		// Free some memory
		sqlFreeResult($result_master);

		// Remember all rows
		$content['rows'] = $OUT;

		// Merge more data again
		$content = merge_array($content, $templateContent);

		// Load main template
		loadTemplate('admin_list_user', FALSE, $content);
	} else {
		// No one as registered so far! :-(
		displayMessage('{--ADMIN_NO_NONE_REGISTERED--}');
	}

	// Free memory
	sqlFreeResult($result_master);
}

// [EOF]
?>
